pub enum PercentageContext {
NotAllowed,
Allowed(Optional<NumericBaseType>),
}Expand description
Dictates whether percentages are allowed in the calculation that is parsed using this context, and whether such percentages have a known “percent hint” (the type that they will eventually resolve to). https://drafts.csswg.org/css-values-4/#calc-context
Variants
NotAllowed
Percentages are not allowed in this calculation context.
Allowed(Optional<NumericBaseType>)
Percentages are allowed with the given pecent hint information.
Implementations
Sourceimpl PercentageContext
impl PercentageContext
pub fn not_allowed() -> Self
pub fn allowed() -> Self
pub fn allowed_with_hint(hint: NumericBaseType) -> Self