pub struct CalcNumeric {
pub clamping_mode: AllowedNumericType,
pub node: CalcNode,
}Expand description
A struct to hold a simplified calc expression and associated clamping mode.
In some cases, e.g. DOMMatrix, we support calc(), but reject all the relative lengths, and to_computed_pixel_length_without_context() handles this case. Therefore, if you want to add a new field, please make sure this function work properly.
Fields
clamping_mode: AllowedNumericTypenode: CalcNodeImplementations
Sourceimpl CalcNumeric
impl CalcNumeric
Sourcepub fn with_clamping_mode(&self, clamping_mode: AllowedNumericType) -> Self
pub fn with_clamping_mode(&self, clamping_mode: AllowedNumericType) -> Self
Returns a new CalcNumeric with the same expression but the specified clamping mode
Sourcepub fn with_leaf_node(&self, leaf: Leaf) -> Self
pub fn with_leaf_node(&self, leaf: Leaf) -> Self
Returns a new CalcNumeric with the same clamping mode but a different leaf node
Sourcepub fn resolve(
&self,
context: &Context<'_>,
leaf_to_f32: impl FnOnce(Result<Leaf, ()>) -> f32,
) -> f32
pub fn resolve( &self, context: &Context<'_>, leaf_to_f32: impl FnOnce(Result<Leaf, ()>) -> f32, ) -> f32
Resolves this calc expression given a computed context, applying clamping.
Sourcepub fn as_number(&self) -> Option<NoCalcNumber>
pub fn as_number(&self) -> Option<NoCalcNumber>
Gets this calc expression as a number
Sourcepub fn as_percentage(&self) -> Option<NoCalcPercentage>
pub fn as_percentage(&self) -> Option<NoCalcPercentage>
Gets this calc expression as a percentage
Sourcepub fn as_time(&self) -> Option<NoCalcTime>
pub fn as_time(&self) -> Option<NoCalcTime>
Gets this calc expression as a time
Sourcepub fn as_resolution(&self) -> Option<NoCalcResolution>
pub fn as_resolution(&self) -> Option<NoCalcResolution>
Gets this calc expression as a resolution
Sourcepub fn as_angle(&self) -> Option<NoCalcAngle>
pub fn as_angle(&self) -> Option<NoCalcAngle>
Gets this calc expression as an angle