pub struct CalcParseFlags {
pub percentage_context: PercentageContext,
pub color_components: ChannelKeyword,
pub additional_functions: AdditionalFunctions,
pub in_place_operations: CalcNodeParseInPlaceOperations,
}Expand description
What is allowed to be parsed for math functions within in this context?
Fields
percentage_context: PercentageContextWhether percentages are allowed in this context, and what numeric type they are relative to. Used both to control parsing as well as to type check calculation trees.
color_components: ChannelKeywordWhich relative color components, if any, are allowed.
additional_functions: AdditionalFunctionsAdditional functions allowed to be parsed in this context.
in_place_operations: CalcNodeParseInPlaceOperationsWhether or not in place operations should be performed. Normally, we aggressive simplify via in-place operations, but it is disabled for generating a trace of steps.
Implementations
Sourceimpl CalcParseFlags
impl CalcParseFlags
Sourcepub fn new(percentage_context: PercentageContext) -> Self
pub fn new(percentage_context: PercentageContext) -> Self
Builds parse flags with the given percentage calculation context.