pub struct CalcParseFlags {
units: CalcUnits,
pub color_components: ChannelKeyword,
additional_functions: AdditionalFunctions,
in_place_operations: CalcNodeParseInPlaceOperations,
}Expand description
What is allowed to be parsed for math functions within in this context?
Fields
units: CalcUnitsUnits allowed to be parsed.
color_components: ChannelKeywordWhich relative color components, if any, are allowed.
additional_functions: AdditionalFunctionsAdditional functions allowed to be parsed in this context.
in_place_operations: CalcNodeParseInPlaceOperationsWhether or not in place operations should be performed. Normally, we aggressive simplify via in-place operations, but it is disabled for generating a trace of steps.
Implementations
Sourceimpl CalcParseFlags
impl CalcParseFlags
Sourcepub fn new(units: CalcUnits) -> Self
pub fn new(units: CalcUnits) -> Self
Allow only specified units to be parsed, without any additional functions
Sourcefn new_including(self, units: CalcUnits) -> Self
fn new_including(self, units: CalcUnits) -> Self
Add new units to the allowed units to be parsed.
Sourcepub fn new_without_in_place_operations(self) -> Self
pub fn new_without_in_place_operations(self) -> Self
Prevents in place operations to be performed